<?php
/**
 * @file
 * Resource callbacks for the Commerce Event Ticket Services module.
 */

/**
 * Validate callback to mark a ticket as used.
 */
function cet_services_validate_ticket($barcode, $nid = NULL, $log = NULL) {
  $ticket = commerce_event_ticket_load_by_barcode($barcode);
  if (!$ticket) {
    return services_error(t('Ticket not found'), 404);
  }
  $output = array();
  $log = empty($log) ? t('Validated via service.') : $log;
  $result = commerce_event_ticket_validate($ticket, TRUE, TRUE, $log);
  if ($result === TRUE) {
    $output['validated'] = TRUE;
  }
  else {
    drupal_add_http_header('Status', '400 ' . t('Ticket not validated'));
    $output['validated'] = FALSE;
    $output['reason'] = $result;
  }
  return $output;
}

/**
 * Retrieve callback to get ticket information based on a barcode.
 */
function cet_services_retrieve_ticket($barcode) {
  $ticket = commerce_event_ticket_load_by_barcode($barcode);
  if (!$ticket) {
    return services_error(t('Ticket not found'), 404);
  }
  $valid = commerce_event_ticket_is_valid($ticket);
  if ($valid) {
    $reason = NULL;
  }
  else {
    $reason = commerce_event_ticket_explain_validity($ticket);
  }
  $output = array(
    'valid' => commerce_event_ticket_is_valid($ticket),
    'reason' => $reason,
    'barcode_token' => $ticket->barcode_token,
    'ticket_id' => $ticket->ticket_id,
    'used' => (bool) $ticket->used,
    'position' => t('@number of @count', array(
      '@number' => $ticket->number,
      '@count' => $ticket->count,
    )),
    'created' => date('c', $ticket->created),
    'changed' => date('c', $ticket->changed),
  );
  $access_all_info = user_access('view all info for a commerce event ticket');
  $order = commerce_order_load($ticket->order_id);
  if ($order && ($access_all_info || commerce_order_access('view', $order))) {
    $output['order'] = array(
      'order_id' => $order->order_id,
      'order_number' => $order->order_number,
      'status' => $order->status,
      'status_title' => commerce_order_status_get_title($order->status),
      'created' => date('c', $order->created),
      'changed' => date('c', $order->changed),
    );
    $owner = user_load($order->uid);
    if ($owner && user_view_access($owner)) {
      $output['order']['owner'] = array(
        'uid' => $order->uid,
        'name' => $owner->name,
        'mail' => $owner->mail,
        'username' => format_username($owner),
        'status' => $owner->status,
      );
    }
  }
  $line_item = commerce_line_item_load($ticket->line_item_id);
  if ($line_item && ($access_all_info || commerce_line_item_access('view', $line_item))) {
    $line_item_wrapper = entity_metadata_wrapper('commerce_line_item', $line_item);
    $line_item_unit_price = $line_item_wrapper->commerce_unit_price->value();
    $line_item_total_price = commerce_line_items_total(array($line_item_wrapper));
    $output['line_item'] = array(
      'line_item_id' => $line_item->line_item_id,
      'type' => $line_item->type,
      'quantity' => $line_item->quantity,
      'unit_price' => commerce_currency_format($line_item_unit_price['amount'], $line_item_unit_price['currency_code']),
      'total_price' => commerce_currency_format($line_item_total_price['amount'], $line_item_total_price['currency_code']),
      'created' => date('c', $line_item->created),
      'changed' => date('c', $line_item->changed),
    );
  }
  $product = commerce_product_load($ticket->product_id);
  if ($product && ($access_all_info || commerce_product_access('view', $product))) {
    $output['product'] = array(
      'product_id' => $ticket->product_id,
      'title' => $product->title,
      'type' => $product->type,
      'status' => $product->status,
      'revision_id' => $product->revision_id,
      'created' => date('c', $product->created),
      'changed' => date('c', $product->changed),
    );
  }
  $node = commerce_event_ticket_get_node($ticket);
  if ($node && ($access_all_info || node_access('view', $node))) {
    $uri = node_uri($node);
    $path = $uri['path'];
    $output['node'] = array(
      'nid' => $node->nid,
      'title' => $node->title,
      'type' => $node->type,
      'status' => $node->status,
      'vid' => $node->vid,
      'created' => date('c', $node->created),
      'changed' => date('c', $node->changed),
      'path_raw' => $GLOBALS['base_path'] . $path,
      'path' => url($path),
    );
  }
  return $output;
}
